import simplejson as json
from django.test import TestCase

from django.conf import settings
from django.utils.importlib import import_module
from shopify_app.services import ShopifyService


class ShopifyServiceTest(TestCase):

    def test_get_all_customers(self):

        objs = ShopifyService(token=settings.SHOPIFY_TEST_PASSWORD, domain=settings.SHOPIFY_TEST_HOST).Customer.find()

        self.assertEquals(type(objs), list)
        self.assertEquals(len(objs), 1)

        self.assertEquals(objs[0].attributes["first_name"], "test")
        self.assertEquals(objs[0].attributes["last_name"], "test")
        self.assertEquals(objs[0].attributes["email"], "test@test.com")

    def test_create_product(self):

        data = {
            "title": "test",
            "body_html": "<strong>test</strong>",
            "vendor": "test",
            "product_type": "test",
            "tags": "test"
        }        

        product = ShopifyService(token=settings.SHOPIFY_TEST_PASSWORD, domain=settings.SHOPIFY_TEST_HOST).Product.create(data)

        self.assertEquals(type(product.id), int)

        for key, value in data.iteritems():
            self.assertEquals(product.attributes[key], value)

    def test_delete_products(self):

        products = ShopifyService(token=settings.SHOPIFY_TEST_PASSWORD, domain=settings.SHOPIFY_TEST_HOST).Product.find()

        for product in products:            
            product.destroy()

        products = ShopifyService(token=settings.SHOPIFY_TEST_PASSWORD, domain=settings.SHOPIFY_TEST_HOST).Product.find()

        self.assertEquals(products, [])


class AnsiColors(object):
    HEADER = '\033[95m'
    OKCYAN = '\033[36m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    WHITEONBLUE = '\033[37;44m'
    WHITE = '\033[37m'

    def disable(self):
        self.HEADER = ''
        self.OKCYAN = ''
        self.OKBLUE = ''
        self.OKGREEN = ''
        self.WARNING = ''
        self.FAIL = ''
        self.ENDC = ''
        self.WHITEONBLUE = ''
        self.WHITE = ''


from django.test import TestCase

from django.conf import settings
from shopify_app.services import ShopService
from shopify_app.services import ShopifyService
from shopify_app.services import ConfigService

from django.core.handlers.base import BaseHandler
from django.test.client import RequestFactory
 

class RequestMock(RequestFactory):

    session = {}

    def request(self, **request):
        """
            Construct a generic request object.
        """

        request = RequestFactory.request(self, **request)
        handler = BaseHandler()
        handler.load_middleware()
        for middleware_method in handler._request_middleware:
            if middleware_method(request):
                raise Exception("Couldn't create request mock object - request middleware returned a response")
        return request


class ShopServiceTest(TestCase):

    config_service = ConfigService()
    shopify_service = ShopifyService(token=settings.SHOPIFY_TEST_PASSWORD, domain=settings.SHOPIFY_TEST_HOST)

    def setUp(self):

        self.request = RequestMock()
        self.request.session["shopify"] = {
            "access_token": settings.SHOPIFY_TEST_PASSWORD,
            "shop_url": settings.SHOPIFY_TEST_HOST,
        }

        self.shopify_service.public_app = True

    def set_billing(self, state):

        config = self.config_service.get_config()
        config.enable_billing = state
        config.save()

    def test_shop_install(self):

        self.set_billing(False)
        shop, redirect_url = ShopService().install(self.request)

        self.assertTrue(shop.id is not None)
        self.assertTrue(redirect_url is False)
        
    def test_shop_billing_install(self):
        
        self.set_billing(True)
        shop, redirect_url = ShopService().install(self.request)

        self.assertTrue(shop.id is not None)
        self.assertTrue(isinstance(redirect_url, basestring))

    def tearDown(self):

        self.shopify_service.public_app = False


from django.test import TestCase

from django.conf import settings
from shopify_app.services import ConfigService


class ShopServiceTest(TestCase):

    def test_shop_install(self):

        config = ConfigService().get_config()
        self.assertEquals(config.id, 1)

    def test_billing_disabled_by_default(self):

        self.assertEquals(ConfigService().is_active_billing(), False)


def slice(objects, prop):
    """
        slice objects in sublist sharing a common propery
    """
    if not objects:
        return []

    def get_prop(obj, prop):
        if not isinstance(objects[0], dict):
            return obj.__dict__[prop]
        return obj[prop]
    
    set_objects = set([get_prop(o, prop) for o in objects])
    return [[o for o in objects if get_prop(o, prop) == obj] for obj in set_objects]


def convert_to_bool(data, params):

    for param in params:
        if not data[param]:
            data[param] = False
        else:
            if data[param] == "on":
                data[param] = True
            else:
                data[param] = bool(int(data[param]))


def normalize_url(url):

    if not url.startswith("http://") and not url.startswith("https://"):
        return "http://%s" % url
    return url


from base import BaseService
from shopify_app.models import Plan
from shopify_service import ShopifyService


class PlanService(BaseService):

    entity = Plan

    def is_active_plan(self, plan):
        """
            Returns if the plan is active or not.
        """

        if plan.uninstalled_at is not None:
            return False

        if plan.billing_type == "O":
            return ShopifyService(shop=plan.shop).is_active_charge(plan.charge_id)

        return ShopifyService(shop=plan.shop).is_active_recurring_charge(plan.charge_id)



from base import BaseView
from shopify_app.config import DEFAULTS
from django.conf import settings

from shopify_app.utils.importer import import_shop_service
from shopify_app.services.log_service import LogService
from shopify_app.services.shopify_api import APIWrapper
from shopify_app.services.shop_service import ShopService


class LoginView(BaseView):
    """
        Initial login action which ask user for their ${shop}.myshopify.com address and self.redirect user to shopify auth page
    """

    def get(self, *args, **kwargs):

        LogService().log_request(self.request)

        if hasattr(self.request, 'session') and 'shopify' in self.request.session:
            return self.redirect(getattr(settings, "PREFERENCES_URL", DEFAULTS["PREFERENCES_URL"]))

        #If the ${shop}.myshopify.com address is already provided in the URL, just skip to authenticate
        if self.request.REQUEST.get('shop'):

            shop = self.request.REQUEST.get('shop').strip()
            permission_url = APIWrapper(shop_url=shop).permissions_url()

            LogService().log_shopify_request(permission_url)

            return self.redirect(permission_url)

        return super(BaseView, self).get(*args, **kwargs)


class FinalizeView(BaseView):
    """
        Finalize login action which receives the request from shopify and login the user to our app.
    """

    service = import_shop_service()

    def get(self, *args, **kwargs):

        LogService().log_request(self.request)

        shop_url = self.request.REQUEST.get('shop')
        shop = self.service.get_shop_by_myshopify_domain(shop_url)

        if shop is None:
            #If shop doesn't exists get the permanent token using the API
            try:
                permanent_token = APIWrapper(shop_url=shop_url).permanent_token(self.request.GET["code"])
            except:
                #Shopify session fails, self.redirect to login initial step
                return self.redirect("%s?shop=%s" % ("/oauth/login", shop_url))
        else:
            permanent_token = shop.token if hasattr(shop, "token") else shop.get_token()

        # Store shopify sesssion data in our session
        self.request.session['shopify'] = {
            "shop_url": shop_url,
            "access_token": permanent_token,
        }

        return self.redirect(getattr(settings, "PREFERENCES_URL", DEFAULTS["PREFERENCES_URL"]))



import os

from django.core.management import call_command
from django.utils.crypto import get_random_string
from templates import render_template
import sys
from ansicolors import AnsiColors

class ProjectBuilder(object):
    """
        A builder for creating the directories structure and files described on:
        https://github.com/SocalProofit/django-shopify/wiki/Getting-started#basic-usage
    """

    def __init__(self, *args):
        """ 
            Setups the project_name and app_name based on *args
            *args are the parameters the user sends through the console command.

            Example:

                #in this case *args would be (project_name, )
                start_shopify_app [project_name]
        """

        self.args = args
        self.project_name = args[0] if self.args else None
        self.app_name = "{0}_app".format(self.project_name)
        self.type = args[1] if len(self.args) > 1 else False
        self.public_app = self.type == "public"

    def build(self):
        """
            Builds the project structure and create files.
        """

        print
        print AnsiColors.WHITEONBLUE + "                                                     " + AnsiColors.ENDC
        print AnsiColors.WHITEONBLUE + "  Django Shopify Core Library - by Bootstrap Heroes  " + AnsiColors.ENDC
        print AnsiColors.WHITEONBLUE + "                                                     " + AnsiColors.ENDC
        print

        if self.project_name == None:
            print AnsiColors.WARNING + "Please provide a project name." + AnsiColors.ENDC
            print "Usage: start_shopify_app [project_name]" + AnsiColors.ENDC
            print
            sys.exit()
        else:  
            self.make_project()
            self.setup_media()
            self.make_default_app()
            
            self.setup_urls()
            self.setup_settings()

            self.go_back_to_main_dir()

            print AnsiColors.OKGREEN + "Shopify Django app " + AnsiColors.WHITE + self.project_name + AnsiColors.OKGREEN + " created! :)" + AnsiColors.ENDC

        print

    def make_project(self):
        """
            Creates the django project and some templates.
        """
 
        call_command("startproject", self.project_name)
        os.chdir(self.project_name)

        self._create_dir("templates")
        self._create_dir("templates", "index")
        self._create_file(self._get_dir("templates", "index", "index.html"), render_template("index.html"))

        if self.public_app:
            self._create_dir("templates", "oauth")
            self._create_file(self._get_dir("templates", "oauth", "login.html"), render_template("login.html"))

    def make_default_app(self):
        """
            Creates the django app, the views root folder and some example views.
        """
        
        print AnsiColors.OKCYAN + "Starting Django app..." + AnsiColors.ENDC

        call_command("startapp", self.app_name)

        os.remove(self._get_app_dir("views.py"))

        self._create_dir(self.app_name, "views")
        self._create_file(self._get_app_dir("views", "__init__.py"), "")
        self._create_file(self._get_app_dir("views", "index.py"), render_template("index"))

        os.chdir(self.project_name)

    def setup_settings(self):
        """
            Replaces the django default settings.py for a template we provide.
        """

        print AnsiColors.OKCYAN + "Configuring " + AnsiColors.WHITE + "settings.py" + AnsiColors.OKCYAN + "..." + AnsiColors.ENDC

        # Create a random SECRET_KEY hash to put it in the main settings.
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        secret_key = get_random_string(50, chars)

        if self.public_app:
            template_name = "settings_public"
        else:
            template_name = "settings"

        settings_file = render_template(template_name, {"app_name": self.app_name, "project_name": self.project_name, "secret_key": secret_key })

        self._create_file("settings.py", settings_file)

    def setup_urls(self):
        """
            Replaces the django default urls.py for a template we provide.
        """

        print AnsiColors.OKCYAN + "Configuring " + AnsiColors.WHITE + "urls.py" + AnsiColors.OKCYAN + "..." + AnsiColors.ENDC

        if self.public_app:
            template_name = "urls_public"
        else:
            template_name = "urls"

        urls_file = render_template(template_name, {"app_name": self.app_name })
        self._create_file("urls.py", urls_file)

    def setup_media(self):
        """
            Creates the media folder
        """

        print AnsiColors.OKCYAN + "Creating media folder..." + AnsiColors.ENDC

        self._create_dir("media")
        self._create_dir("media", "js")
        self._create_dir("media", "css")
        self._create_dir("media", "img")

    def go_back_to_main_dir(self):
        """
            After creating the structure this goes back to the directory where we started
            before calling os.chdir in order to create some dirs and files inside the tree.
        """

        os.chdir(os.pardir)
        os.chdir(os.pardir)

    def _create_file(self, name, content):
        """
            Created a file with [content]
        """

        with open(name, "w") as f:
            f.write(content)

    def _get_app_dir(self, *args):
        """
            Returns the app directory
        """

        return os.path.join(self.app_name, *args)

    def _get_dir(self, *args):
        """
            Returns a directory using os.path.join
        """

        return os.path.join(*args)

    def _create_dir(self, *args):
        """
            Creates a new dir joining *args
        """

        os.mkdir(self._get_dir(*args))


import os.path

__dir__ = os.path.dirname(os.path.abspath(__file__))
templates_dir = os.path.join(__dir__, os.pardir, "templates")


def render_template(template_name, context=None):
    """
        Renders a template located on the /management/commands/templates
        directory.
    """

    if context is None:
        context = {}

    if not template_name.endswith(".tm"):
        template_name = "{0}.tm".format(template_name)

    with open(os.path.join(templates_dir, template_name), "r") as f:
        content = f.read()

    return content.format(**context)


from base import BaseService
from shopify_app.models import Config

from plan_config_service import PlanConfigService


class ConfigService(BaseService):

    entity = Config

    def get_config(self):
        """
            Gets the config object.
            Created a new default config if it doesn't exists.
        """ 

        #lazy attribute
        if not "config" in self.__dict__:

            self.config = self.get_one(id=1)

            if self.config is None:

                self.config = self.new(id=1)

                if self.config.plan_config is None:

                    plan_config = PlanConfigService().new()
                    plan_config.save()

                    self.config.plan_config = plan_config

                self.config.save()

        return self.config

    def is_active_billing(self):
        """
            Returns if the billing is enabled or not
        """

        config = self.get_config()
        return config is not None and config.enable_billing


from django.shortcuts import redirect
from django.conf import settings
from shopify_app.services.shopify_api import APIWrapper


def shop_login_required(func):

    def wrapper(view, request, *args, **kwargs):

        if not settings.SHOPIFY_API_KEY or not settings.SHOPIFY_API_SECRET:
            raise Exception("SHOPIFY_API_KEY and SHOPIFY_API_SECRET must be set in settings")

        if hasattr(request, 'session') and 'shopify' in request.session:

            #Check if the app was uninstalled
            try:
                api_wrapper = APIWrapper(token=request.session["shopify"]["access_token"], shop_url=request.session["shopify"]["shop_url"])
                api_wrapper.current_shop()
            except:
                request.session.pop("shopify", None)
                return redirect("/oauth/login")
        else:
            request.session['return_to'] = request.get_full_path()
            return redirect("/oauth/login")

        return func(view, request, *args, **kwargs)

    wrapper.__name__ = func.__name__
    return wrapper



import time

from shopify_app.services.log_service import LogService
from shopify_app.config import DEFAULTS

from django.http import HttpResponseServerError
from shopify_app.utils.email import send_html_email

from django.conf import settings


def handler500(request):

    log = LogService().log_error(request)

    subject = "An error happened with id %s" % log.id

    sender = getattr(settings, "ERROR_EMAIL_SENDER", DEFAULTS["ERROR_EMAIL_SENDER"])
    receiver = getattr(settings, "ERROR_EMAIL_RECEIVER", DEFAULTS["ERROR_EMAIL_RECEIVER"])

    if sender is not None and receiver is not None:
        send_html_email(subject, log.stack_trace, sender, receiver)

    return HttpResponseServerError("<h1>500. Ups... An error ocurred</h1>")


from django.core.mail import EmailMultiAlternatives
from threading import Thread


def send_html_email(subject, content, from_email, to_email, fail_silently=False, bcc=[]):

    if not isinstance(to_email, list):
        to_email = [to_email]

    msg = EmailMultiAlternatives(subject=subject, body=content, from_email=from_email, to=to_email, bcc=bcc)
    msg.attach_alternative(content, "text/html")

    thread = Thread(target=lambda: msg.send(fail_silently=fail_silently))
    thread.start()


from django.core.management.base import BaseCommand, CommandError
from optparse import make_option

from lib.builder import ProjectBuilder

class Command(BaseCommand):

    def handle(self, *args, **options):
        self.do_run(*args)
    
    def do_run(self, *args):
        builder = ProjectBuilder(*args)
        builder.build()


from base import BaseView
from django.conf import settings
from shopify_app.config import DEFAULTS

from shopify_app.utils.importer import import_shop_service
from shopify_app.decorators import shop_login_required
from shopify_app.services.shop_service import ShopService
from shopify_app.services.log_service import LogService


class BaseShopView(BaseView):

    service = import_shop_service()


class IndexView(BaseShopView):

    def get(self, *args, **kwargs):

        return self.redirect(getattr(settings, "PREFERENCES_URL", DEFAULTS["PREFERENCES_URL"]))


class PreferencesView(BaseShopView):
    """
        Main View to access the shop account
    """

    @shop_login_required
    def get(self, *args, **kwargs):

        LogService().log_request(self.request)

        shop, redirect_url = self.service.install(self.request)
        self.request.session["shop"] = shop

        if not redirect_url:
            redirect_url = getattr(settings, "OAUTH_REDIRECT_URL", DEFAULTS["OAUTH_REDIRECT_URL"])

        return self.redirect(redirect_url)


class BillingView(BaseShopView):
    """
        Return handler for when a user accepts the billing plan for the app
    """

    @shop_login_required
    def get(self, *args, **kwargs):

        LogService().log_request(self.request)

        shop_id = self.request.GET.get("shop")
        plan_config_id = self.request.GET.get("plan_config")
        charge_id = self.request.GET.get("charge_id")

        if not ShopService().upgrade_plan(shop_id, plan_config_id, charge_id):
            return self.redirect(getattr(settings, "BILLING_DECLINE_REDIRECT_URL", DEFAULTS["BILLING_DECLINE_REDIRECT_URL"]))

        return self.redirect(getattr(settings, "BILLING_REDIRECT_URL", DEFAULTS["BILLING_REDIRECT_URL"]))



# -*- coding: utf-8 -*-
from django.shortcuts import get_object_or_404

from shopify_app.utils.render import render, render_string
from shopify_app.utils.python import convert_to_bool


class BaseService(object):

    _repo = property(fget=lambda self: self.entity.objects)
    _page_size = 10

    default_query_params = {}

    def __getattr__(self, name):
        """
            Delegates automatically all undefined methods on the repository entity.            
        """

        def decorator(*args, **kwargs):

            method = getattr(self._repo, name)
            if method is None:
                raise AttributeError("'%s' has no attribute '%s'" % (self.__class__.__name__, name))

            if not kwargs.pop("without_filters", False):
                for key, value in self.default_query_params.iteritems():
                    kwargs.setdefault(key, value)

            return method(*args, **kwargs)

        return decorator

    def get_page(self, page=0, size=None, min_page=None, **kwargs):

        if size is None:
            size = self._page_size

        page = int(page)

        if min_page is not None:
            min_page = int(min_page)
            limit = (page + 1) * size
            offset = min_page * size
        else:
            limit = (page + 1) * size
            offset = size * page

        return self._get_objects(self._get_page_query(offset, limit, **kwargs))

    def _get_page_query(self, offset, limit, **kwargs):

        return self.all()[offset:limit]

    def list(self, start, size, **kwargs):
        page = int(start / size)
        return self.get_page(page=page, size=size, min_page=None, **kwargs)

    def _get_objects(self, objects):
        """ Override to add behaviour """

        return objects

    def get_one(self, *args, **kwargs):

        objects = self.filter(*args, **kwargs)
        return objects[0] if objects else None

    def new(self, *args, **kwargs):

        return self.entity(*args, **kwargs)

    def _get_or_new(self, *args, **kwargs):

        try:
            obj, created = self.get_or_create(*args, **kwargs)
        except:
            obj, created = self.entity(*args, **kwargs), True
        return obj, created

    def get_or_new(self, *args, **kwargs):

        obj, _ = self._get_or_new(*args, **kwargs)
        return obj

    def update_or_create(self, pre_create_function=None, pre_update_function=None, *args, **kwargs):

        entity_id = kwargs.pop("id", None)
        if entity_id:
            
            if pre_update_function is not None:
                pre_update_function(kwargs)

            entity = self.get(id=entity_id)
            for key, value in kwargs.iteritems():
                setattr(entity, key, value)
            
        else:
            if pre_create_function is not None:
                pre_create_function(kwargs)

            entity = self.new(**kwargs)

        entity.save()
        return entity

    def get_or_new_created(self, *args, **kwargs):

        return self._get_or_new(*args, **kwargs)

    def get_form(self):

        return None

    def _get_data(self, request, *args, **kwargs):

        data = dict([(key, value) for key, value in request.POST.iteritems() if key != "csrfmiddlewaretoken"])
        data.update(self._get_additional_data(request))
        return data

    def _get_additional_data(self, request, *args, **kwargs):

        return {}

    def _get_entity(self, request, *args, **kwargs):

        return self.get_or_new(**self._get_data(request))

    def _set_data(self, entity, request, *args, **kwargs):

        data = self._get_data(request)
        for key, value in data.iteritems():
            setattr(entity, key, value)
        return entity

    def set_attrs(self, entity, attrs):

        for key, value in attrs.iteritems():
            setattr(entity, key, value)

    def save_entity(self, entity, *args, **kwargs):

        entity.save()

    def save(self, request, *args, **kwargs):

        entity = self._get_entity(request, *args, **kwargs)

        self._set_data(entity, request, *args, **kwargs)
        self.save_entity(entity, *args, **kwargs)
        self._post_save(entity, request, *args, **kwargs)

        return entity

    def _post_save(self, entity, request, *args, **kwargs):

        pass

    def render(self, template, context):

        return render(template, context)

    def render_string(self, string, context):

        return render_string(string, context)

    def get_object_or_404(self, **kwargs):

        return get_object_or_404(self.entity, **kwargs)

    def delete(self, *args, **kwargs):

        logical_delete = kwargs.pop("logical", False)

        objs = self.filter(*args, **kwargs)

        if not objs:
            return False

        for obj in objs:
            if not logical_delete:
                obj.delete()
            else:
                obj.active = False
                obj.save()

        return True

    def get_formated_sum(self, value):

        if value is None:
            value = 0

        return "%.2f" % value

    def _render_row_value(self, row_data, render):

        if isinstance(render, basestring):
            if isinstance(row_data, dict):
                return unicode(row_data[render])
            else:
                return unicode(getattr(row_data, render))
        else:
            return unicode(render(row_data))

    def get_params(self, data, params):

        dict_params = {}
        for param in params:
            dict_params[param] = data.get(param)
        return dict_params

    def convert_to_bool(self, data, params):

        convert_to_bool(data, params)

    def to_bool(self, param):

        return bool(int(param))

    def get_action_params(self, request, params_names, prefix="", bar_action=True):

        complete_names = ["%s%s" % (prefix, param) for param in params_names]

        params = self.get_params(request.POST, complete_names)

        if bar_action:
            boolean_params = ["%s%s" % (prefix, param) for param in ["is_main_action", "is_side_action"]]
            self.convert_to_bool(params, boolean_params)

        final_params = {}
        for key, value in params.iteritems():
            new_key = key.replace(prefix, "")
            final_params[new_key] = value

        return final_params

    def check_nullables(self, data, params):

        for param in params:
            if not data.get(param):
                data[param] = None



from datetime import datetime
from django.db import models


class BaseEntity(models.Model):

    NOT_IN_FIELDS = []

    created_at = models.DateTimeField(null=True, blank=True)
    updated_at = models.DateTimeField(null=True, blank=True)

    def __init__(self, *args, **kwargs):

        now = datetime.now()
        kwargs.update(created_at=now, updated_at=now)
        models.Model.__init__(self, *args, **kwargs)

    def fields(self):

        return [key for key in self.__dict__.keys() if not key.startswith("_")]

    def update_fields(self):

        return [field for field in self.fields() if field not in self.NOT_IN_FIELDS]

    class Meta:
        abstract = True


class Shop(BaseEntity):
    """
        Represents a Store with all the information stored on shopify API
    """
    shop_id = models.CharField(max_length=255, null=False, blank=False, unique=True)
    token = models.CharField(max_length=255, null=True, blank=True)

    address1 = models.CharField(max_length=255, null=True, blank=True)
    city = models.CharField(max_length=255, null=True, blank=True)
    country = models.CharField(max_length=255, null=True, blank=True)
    country_code = models.CharField(max_length=255, null=True, blank=True)
    country_name = models.CharField(max_length=255, null=True, blank=True)
    customer_email = models.CharField(max_length=255, null=True, blank=True)
    currency = models.CharField(max_length=255, null=True, blank=True)
    domain = models.CharField(max_length=255, null=True, blank=True)
    email = models.CharField(max_length=255, null=True, blank=True)
    google_apps_domain = models.CharField(max_length=255, null=True, blank=True)
    google_apps_login_enabled = models.CharField(max_length=255, null=True, blank=True)
    latitude = models.CharField(max_length=255, null=True, blank=True)
    longitude = models.CharField(max_length=255, null=True, blank=True)
    money_format = models.CharField(max_length=255, null=True, blank=True)
    money_with_currency_format = models.CharField(max_length=255, null=True, blank=True)
    myshopify_domain = models.CharField(max_length=255, null=True, blank=True)
    name = models.CharField(max_length=255, null=True, blank=True)
    plan_name = models.CharField(max_length=255, null=True, blank=True)
    display_plan_name = models.CharField(max_length=255, null=True, blank=True)
    phone = models.CharField(max_length=255, null=True, blank=True)
    province = models.CharField(max_length=255, null=True, blank=True)
    province_code = models.CharField(max_length=255, null=True, blank=True)
    public = models.CharField(max_length=255, null=True, blank=True)
    shop_owner = models.CharField(max_length=255, null=True, blank=True)
    source = models.CharField(max_length=255, null=True, blank=True)
    tax_shipping = models.CharField(max_length=255, null=True, blank=True)
    taxes_included = models.CharField(max_length=255, null=True, blank=True)
    county_taxes = models.CharField(max_length=255, null=True, blank=True)
    timezone = models.CharField(max_length=255, null=True, blank=True)
    zip = models.CharField(max_length=255, null=True, blank=True)

    NOT_IN_FIELDS = ["id", "created_at", "updated_at", "shop_id", "token"]

    def current_plan(self):

        plans = self.plan_set.order_by("-id")
        if not plans:
            return

        return plans[0]

    def __unicode__(self):

        return "{0} - {1}".format(self.shop_id, self.domain)


class PlanAttributes(BaseEntity):
    """
        Represents the plan attributes needed to bill stores
    """
    BILLING_TYPE = (
        ('O', 'One Time'),
        ('I', 'Interval'),
    )

    name = models.CharField(max_length=255, null=True, blank=True)
    active = models.BooleanField(default=True)

    billing_amount = models.DecimalField(max_digits=15, decimal_places=4, null=True, blank=True)
    billing_type = models.CharField(max_length=2, null=True, blank=True, choices=BILLING_TYPE)

    trial_period_days = models.PositiveIntegerField(null=True, blank=True)

    class Meta:
        abstract = True

    def __unicode__(self):

        return "{0} - ${1}".format(self.name, self.billing_amount)


class PlanConfig(PlanAttributes):
    """
        Represents the payments configuration for bill shops
    """
    NOT_IN_FIELDS = ["id", "created_at", "updated_at"]


class Plan(PlanAttributes):
    """
        Represents the agreement associated with a shop based on the plan attributes.
    """
    shop = models.ForeignKey("Shop")
    charge_id = models.CharField(max_length=255, null=False, blank=False)

    installed_at = models.DateTimeField(null=True, blank=True)
    uninstalled_at = models.DateTimeField(null=True, blank=True)


class Config(BaseEntity):
    """
        Represents the payment configuration for all shops specified by the admin.
    """
    enable_billing = models.BooleanField(default=False)

    plan_config = models.OneToOneField("PlanConfig", null=True, blank=True)

    def __unicode__(self):

        return "{0}".format(self.id)


class RequestLog(BaseEntity):

    url = models.CharField(max_length=1000, null=True, blank=True)
    headers = models.TextField(null=True, blank=True)
    method = models.CharField(max_length=20, null=True, blank=True)
    payload = models.TextField(null=True, blank=True)
    params = models.TextField(null=True, blank=True)

    response = models.TextField(null=True, blank=True)


class ErrorLog(models.Model):

    page_url = models.CharField(max_length=4000)
    params = models.CharField(max_length=4000)
    headers = models.CharField(max_length=4000)
    stack_trace = models.TextField()

    datetime = models.DateTimeField(default=None, null=True, blank=True)



from django.conf.urls import patterns, include, url

from django_conventions import UrlsManager
import shopify_app.views as views_root

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Uncomment the next line to enable the admin:
    # url(r'^admin/', include(admin.site.urls)),
)

UrlsManager(urlpatterns, views_root)



from urllib import urlencode

import requests
import simplejson as json

from log_service import LogService

from django.conf import settings


class APIWrapper(object):

    def __init__(self, shop=None, token=None, api_domain=None, shop_url=None, log=True):

        if shop is not None:
            token = shop.token
            api_domain = shop.myshopify_domain

        if shop_url is not None:
            shop_url = self._normalize_shop_url(shop_url)

        if not api_domain:
            self.api_domain = "%s/admin" % (shop_url, )
        else:
            self.api_domain = "https://%s/admin" % api_domain

        self.log = log
        self.shop_url = shop_url

        self.params = {
            "headers": {
                'X-Shopify-Access-Token': token,
                'Content-type': 'application/json',
            }
        }

    def _decode_response(self, response):

        try:
            response_json = json.loads(response.text)
        except:
            return

        if "errors" in response_json:
            raise Exception(response_json["errors"])

        return response_json

    def _make_request(self, url, method, params):

        response = getattr(requests, method)(url, **params)

        if self.log and getattr(settings, "LOG_SHOPIFY_API_REQUESTS", False):
            LogService().log_shopify_request(url, method=method, params=params, response=response.text)

        return response

    def _pluralize_entity(self, entity):

        if not entity.endswith("s"):
            return "%ss" % entity
        return entity

    def _return_entity(self, response, entity):

        decoded_response = self._decode_response(response)

        if decoded_response is not None:
            return decoded_response.get(entity)

    def create(self, entity, data):

        params = self.params.copy()
        params["data"] = json.dumps({entity: data})

        pluralized_entity = self._pluralize_entity(entity)

        url = "%s/%s.json" % (self.api_domain, pluralized_entity)
        response = self._make_request(url, "post", params)

        return self._return_entity(response, entity)


    def update(self, id, entity, data):

        params = self.params.copy()
        params["data"] = json.dumps({entity: data})

        pluralized_entity = self._pluralize_entity(entity)

        url = "%s/%s/%s.json" % (self.api_domain, pluralized_entity, id)
        response = self._make_request(url, "put", params)

        return self._return_entity(response, entity)

    def search(self, entity, filters):

        if isinstance(filters, dict):
            filters = urlencode(filters)

        entity = self._pluralize_entity(entity)

        url = "%s/%s/search.json?query=%s" % (self.api_domain, entity, filters)
        response = self._make_request(url, "get", self.params)

        return self._return_entity(response, entity)

    def find(self, entity, filters):

        if isinstance(filters, dict):
            filters = urlencode(filters)

        entity = self._pluralize_entity(entity)

        url = "%s/%s.json?%s" % (self.api_domain, entity, filters)
        response = self._make_request(url, "get", self.params)

        response = self._decode_response(response)
        if entity in response:
            return response[entity]
        else:
            return response

    def get(self, entity, id):

        if isinstance(id, dict):
            id = id.get("id")

        pluralized_entity = self._pluralize_entity(entity)

        url = "%s/%s/%s.json" % (self.api_domain, pluralized_entity, id)
        response = self._make_request(url, "get", self.params)

        return self._return_entity(response, entity)

    def activate_charge(self, entity, charge_id):

        entity = self._pluralize_entity(entity)

        url = "%s/%s/%s/activate.json" % (self.api_domain, entity, charge_id)
        response = self._make_request(url, "post", self.params)

        return self._decode_response(response)

    def _normalize_shop_url(self, shop_url):

        if not shop_url.startswith("http://") and not shop_url.startswith("https://"):
            shop_url = "https://%s" % shop_url

        return shop_url

    def permanent_token(self, code):

        url = "%s/oauth/access_token" % (self.api_domain, )

        params = self.params.copy()
        params["params"] = {
            "code": code,
            "client_id": settings.SHOPIFY_API_KEY,
            "client_secret": settings.SHOPIFY_API_SECRET,
        }

        response = self._make_request(url, "post", params)
        decoded_response = self._decode_response(response)

        return decoded_response["access_token"]

    def permissions_url(self):

        params = {
            "scope": ",".join(settings.SHOPIFY_API_SCOPE),
            "client_id": settings.SHOPIFY_API_KEY,
        }

        return "%s/oauth/authorize?%s" % (self.api_domain, urlencode(params))

    def current_shop(self):

        url = "%s/shop.json" % (self.api_domain, )
        response = self._make_request(url, "get", self.params)

        return self._return_entity(response, "shop")



DEFAULTS = {
    "HOST": "http://127.0.0.1:8000",
    "BILLING_REDIRECT_URL": "/",
    "BILLING_DECLINE_REDIRECT_URL": "/",
    "OAUTH_REDIRECT_URL": "/",
    "PREFERENCES_URL": "/shop/preferences/",
    "ERROR_EMAIL_SENDER": None,
    "ERROR_EMAIL_RECEIVER": None,
}



import importlib
from django.conf import settings

from shopify_app.services.shop_service import ShopService


def import_shop_service():

    shop_service_class_name = getattr(settings, "SHOP_SERVICE", None)
    if shop_service_class_name is None:
        return ShopService()

    shop_service_class_parts = shop_service_class_name.split(".")
    shop_service_pkg = ".".join(shop_service_class_parts[0:-1])

    shop_service_module = importlib.import_module(shop_service_pkg)
    shop_service_class = getattr(shop_service_module, shop_service_class_parts[-1])

    return shop_service_class()



try:
    import shopify
except ImportError:
    pass

from django.conf import settings

from shopify_api import APIWrapper
from shopify_app.utils.python import normalize_url


class ShopifyService(object):
    """
        @Deprecated. Please use services.shopify_api.APIWrapper to make
        requests to the shopify api.
    """

    #Set to True to force public app
    public_app = False

    def __new__(cls, *args, **kwargs):
        """
            Singleton class
        """
        if not hasattr(cls, "_instance"):
            cls._instance = super(ShopifyService, cls).__new__(cls)
        return cls._instance

    def _init_public_app(self, token, domain):
        """
            Initializes the shopify api client with the key of the shop
        """

        try:
            session = shopify.Session(domain)
            session.token = token
            return session
        except AttributeError:
            raise Exception("You have to specify the token and domain to init a public app")

    def _init_private_app(self):
        """
            Initializes the shopify api client with the keys stored in settings.py.
            SHOPIFY_API_KEY, SHOPIFY_API_SECRET, SHOPIFY_API_PASSWORD, and SHOPIFY_HOST
            must be defined in that file.
        """
        try:
            api_key = settings.SHOPIFY_API_KEY
            api_secret = settings.SHOPIFY_API_SECRET
            api_host = settings.SHOPIFY_HOST
            api_password = settings.SHOPIFY_API_PASSWORD
        except AttributeError:
            raise Exception("You have to specify the SHOPIFY_API_KEY, SHOPIFY_API_SECRET, SHOPIFY_API_PASSWORD, and SHOPIFY_HOST in settings.py")

        session = shopify.Session(normalize_url(api_host))
        session.token = api_password
        session.setup(api_key=api_key, secret=api_secret)
        return session

    def __init__(self, token=None, domain=None, shop=None):

        if getattr(settings, "PUBLIC_APP", False) or self.public_app:

            if shop:
                token = shop.token
                domain = shop.myshopify_domain

            session = self._init_public_app(token, domain)

        else:
            session = self._init_private_app()

        shopify.ShopifyResource.activate_session(session)
        self.session = session
        self.shop = shop

    def __getattr__(self, name):
        """
            Delegates all the undefined methods and attributes to the shopify client.
        """

        return getattr(shopify, name)

    def is_active_charge(self, charge_id):

        charge = APIWrapper(self.shop).get("application_charge", charge_id)
        return charge and charge["status"] == "active"

    def is_active_recurring_charge(self, charge_id):

        charge = APIWrapper(self.shop).get("recurring_application_charge", charge_id)
        return charge and charge["status"] == "active"



import os
from shopify_app.management.commands.start_shopify_app import Command

from django.test import TestCase
from shopify_app.services import ShopifyService

import shutil

class CommandsTest(TestCase):

    def clean(self, path):

        try:
            shutil.rmtree(path)
        except:
            pass

    def _get_dir(self, *args):

        return os.path.join(*args)

    def test_start_app(self):

        test_name = "project_testing"
        app_name = "{0}_app".format(test_name)

        self.clean(test_name)

        Command().do_run(test_name)        
        
        self.assert_exists([test_name])
        self.assert_exists([test_name, "templates"])
        self.assert_exists([test_name, "media"])
        self.assert_exists([test_name, app_name])
        self.assert_exists([test_name, "manage.py"], is_dir=False)

        self.assert_exists([test_name, test_name])
        self.assert_exists([test_name, test_name, "settings.py"], is_dir=False)
        self.assert_exists([test_name, test_name, "urls.py"], is_dir=False)
        self.assert_exists([test_name, test_name, "wsgi.py"], is_dir=False)

        self.assert_exists([test_name, app_name, "views"])
        self.assert_exists([test_name, app_name, "views", "index.py"], is_dir=False)

        self.clean(test_name)

    def assert_exists(self, dir_parts, is_dir=True):
        
        assert_func = "isdir" if is_dir else "isfile"

        self.assertTrue(getattr(os.path, assert_func)(self._get_dir(*dir_parts)))        



from django.views.generic import TemplateView
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseServerError, Http404
import simplejson as json

from django.views.decorators.cache import cache_page
from django.utils.decorators import method_decorator

from shopify_app.utils.render import render, render_string
from shopify_app.utils.python import convert_to_bool


class BaseView(TemplateView):

    csrf_exempt = True

    def dispatch(self, request, *args, **kwargs):

        return TemplateView.dispatch(self, request, *args, **kwargs)

    def redirect(self, url):

        return HttpResponseRedirect(url)

    def response(self, response, no_cache=False, headers={}):
        
        http_response = HttpResponse(response)

        if no_cache:
            http_response["Cache-Control"] = "max-age=0"

        for key, value in headers.iteritems():
            http_response[key] = value
            
        return http_response

    def response_error(self, response):

        return HttpResponseServerError(response)

    def response_404(self):

        raise Http404

    def json_response(self, response):

        return self.response(json.dumps(response))

    def json_loads(self, data):

        return json.loads(data)

    def json_dumps(self, data):

        return json.dumps(data)

    def render(self, template, context):

        return render(template, context)

    def render_string(self, string, context):

        return render_string(string, context)

    def get_list_args(self, startswith):

        return [key[len(startswith):] for key, value in self.request.POST.iteritems() if key.startswith(startswith)]

    def export(self, format, filename, content):

        response = HttpResponse(mimetype='text/%s' % format)
        response['Content-Disposition'] = 'attachment; filename=%s.%s' % (filename, format)
        Exporter().export(format, response, content)
        return response

    def get_params(self, data, params):

        dict_params = {}
        for param in params:
            dict_params[param] = data.get(param)
        return dict_params

    def convert_to_bool(self, data, params):

        convert_to_bool(data, params)


class AjaxBaseView(BaseView):

    def on_success(self):

        return self.json_response({"status": "ok"})

    def on_fail(self, error):

        return self.json_response({"status": "fail", "error": error})


class AjaxSaveBaseView(AjaxBaseView):

    def after_save(self, entity):

        pass

    def post(self, *args, **kwargs):

        try:
            self.entity = self.service.save(self.request)
            self.after_save(self.entity)
            return self.render(self.on_success)

        except ValidationError, e:
            return self.render(self.on_fail, str(e))

    def render(self, function, *args, **kwargs):

        return self.response(function(*args, **kwargs))


class AjaxFormSaveBaseView(AjaxSaveBaseView):

    def get_data(self, request):

        return request.POST

    def post(self, *args, **kwargs):

        form = self.form(self.get_data(self.request), *args, **kwargs)

        if (form.is_valid()):
            form.save()
            return self.render(self.on_success)

        return self.render(self.on_error, **form.errors)

    def on_error(self, **data):

        return self.json_response(data)


class CachedView(BaseView):

    @method_decorator(cache_page(60 * 60))
    def dispatch(self, request, *args, **kwargs):
        return super(BaseView, self).dispatch(request, *args, **kwargs)



from django.contrib import admin
from shopify_app.models import Config, PlanConfig, Plan, Shop, ErrorLog, RequestLog


class BaseAdmin(admin.ModelAdmin):

    exclude = ["created_at", "updated_at"]


admin.site.register(Config, BaseAdmin)
admin.site.register(PlanConfig, BaseAdmin)
admin.site.register(Plan, BaseAdmin)
admin.site.register(Shop, BaseAdmin)
admin.site.register(RequestLog, BaseAdmin)
admin.site.register(ErrorLog, admin.ModelAdmin)


import sys
from datetime import datetime
from django.views.debug import ExceptionReporter

from base import BaseService
from shopify_app.models import RequestLog, ErrorLog


class LogService(BaseService):

    entity = RequestLog

    def log_request(self, request):

        data = {
            "url": request.get_full_path(),
            "headers": str(request.META),
            "payload": request.body,
            "method": request.method,
        }

        data["params"] = str(request.REQUEST)

        log = self.new(**data)
        log.save()

    def log_shopify_request(self, url, method="get", params=None, response=None):

        data = {
            "url": url,
            "method": method,
        }

        if response:
            data["response"] = response

        if params:
            data["params"] = str(params)

        log = self.new(**data)
        log.save()

    def log_error(self, request):

        etype, evalue, etraceback = sys.exc_info()
        sys.exc_clear()

        stack_trace = ExceptionReporter(request, etype, evalue, etraceback).get_traceback_text()

        data = {
            "page_url": request.get_full_path(),
            "params": str(request.REQUEST),
            "headers": str(request.META),
            "stack_trace": stack_trace,
            "datetime": datetime.now(),
        }

        log = ErrorLog(**data)
        log.save()

        return log


from base import BaseService
from shopify_app.models import PlanConfig
from django.conf import settings
from shopify_app.config import DEFAULTS
from datetime import datetime
from shopify_api import APIWrapper


class PlanConfigService(BaseService):

    entity = PlanConfig

    def _get_charge_common_data(self, shop, plan_config):
        """
            Returns the common data for the charge API call
        """
        data = {
            "name": plan_config.name if plan_config.name else "Default",
            "price": plan_config.billing_amount if plan_config.billing_amount else 10.0,
            "return_url": "http:%s/shop/billing/?shop=%s&plan_config=%s" % (getattr(settings, "HOST", DEFAULTS["HOST"]), shop.id, plan_config.id),
        }

        if getattr(settings, "TEST", True):
            data["test"] = True

        return data

    def _create_charge(self, shop_model, api_entity, data):

        return APIWrapper(shop_model, log=True).create(api_entity, data)

    def one_time_charge(self, shop, plan_config):
        """
            Generates a one time charge for this app
        """

        data = self._get_charge_common_data(shop, plan_config)

        return self._create_charge(shop, "application_charge", data)

    def recurring_charge(self, shop, plan_config):
        """
            Generates a recurring charge for this app
        """

        data = self._get_charge_common_data(shop, plan_config)
        default_trial_days = plan_config.trial_period_days if plan_config.trial_period_days else 15

        #trial days starts counting from the first install
        current_trial_days = (datetime.utcnow().replace(tzinfo=None) - shop.created_at.replace(tzinfo=None)).days
        if not current_trial_days >= default_trial_days:
            data["trial_days"] = default_trial_days - current_trial_days

        return self._create_charge(shop, "recurring_application_charge", data)

    def confirm_data(self, shop, plan_config):
        """
            Makes the request to Generate either a one time charge or recurring charge and
            returns the response results.

            If there are errors in the request response it raises an exception.
        """

        if plan_config.billing_type == "O":
            response = self.one_time_charge(shop, plan_config)
        else:
            response = self.recurring_charge(shop, plan_config)

        if "errors" in response:
            raise Exception(str(response["errors"]))

        return response



from django_webtest import WebTest
from django.conf import settings
from django.utils.importlib import import_module
from shopify_app.models import Shop
from shopify_app.services import ConfigService

class ShopView(WebTest):

    def setUp(self):
        settings.SESSION_ENGINE = 'django.contrib.sessions.backends.file'
        engine = import_module(settings.SESSION_ENGINE)
        store = engine.SessionStore()
        store.save()
        self.session = store
        self.client.cookies[settings.SESSION_COOKIE_NAME] = store.session_key

        shop = Shop.objects.filter(myshopify_domain=settings.SHOPIFY_TEST_HOST)
        if shop:
        	shop.delete()

    def test_not_logged_preferences(self):
        response = self.app.get('/shop/preferences/')

        assert response.status_code == 302
        assert '/oauth/login' in response.location
        #FIXME
        #self.assertRedirects(response, '/oauth/login', status_code=301)

    def test_install_preferences(self):

        session = self.client.session
        session["shopify"] = {
            "access_token": settings.SHOPIFY_TEST_PASSWORD,
            "shop_url": settings.SHOPIFY_TEST_HOST,
        }
        session.save()

        # pretend to be logged in as user `kmike` and go to the index page
        response = self.client.get('/shop/preferences/')

        assert response.status_code == 302
        assert Shop.objects.filter(myshopify_domain=settings.SHOPIFY_TEST_HOST).exists()

        # pretend to be logged in as user `kmike` and go to the index page
        response = self.client.get('/shop/preferences/')

        assert response.status_code == 302
        assert len(Shop.objects.filter(myshopify_domain=settings.SHOPIFY_TEST_HOST))== 1

    def test_install_with_payments(self):
        config = ConfigService().get_config()
        config.enable_billing = True
        config.save()

        session = self.client.session
        session["shopify"] = {
            "access_token": settings.SHOPIFY_TEST_PASSWORD,
            "shop_url": settings.SHOPIFY_TEST_HOST,
        }
        session.save()

        # pretend to be logged in as user `kmike` and go to the index page
        response = self.client.get('/shop/preferences/')

        assert response.status_code == 302
        assert "%s/admin/charges/"%settings.SHOPIFY_TEST_HOST in response["location"]
        assert Shop.objects.filter(myshopify_domain=settings.SHOPIFY_TEST_HOST).exists()        

        config.enable_billing = False
        config.save()



from base import BaseService

from shopify_app.models import Shop

from shopify_service import ShopifyService
from config_service import ConfigService
from plan_service import PlanService
from log_service import LogService
from shopify_api import APIWrapper
from plan_config_service import PlanConfigService
from datetime import datetime


class ShopService(BaseService):

    entity = Shop

    def get_shop_by_myshopify_domain(self, shop_url):

        return self.get_one(myshopify_domain=shop_url)

    def install(self, request):
        """
            Installation / app preferences service handler.

            Creates a new store object if it doesn't exists with
            all the shopify API shop attributes.
        """

        self.before_install(request)

        token = request.session.get('shopify', {}).get("access_token")
        domain = request.session.get('shopify', {}).get("shop_url")

        shop = APIWrapper(token=token, api_domain=domain, log=True).current_shop()
        shop_model, created = self.get_or_create(shop_id=shop["id"])

        for field in shop_model.update_fields():
            setattr(shop_model, field, shop.get(field))

        shop_model.token = token

        self.before_save(shop_model, request)

        shop_model.save()

        self.post_install(request)

        if not self._check_active_plan(shop_model):

            redirect_url = self.get_upgrade_plan_url(shop_model)
            LogService().log_shopify_request(redirect_url)

            return shop_model, redirect_url

        return shop_model, False

    def _check_active_plan(self, shop):
        """
            This method checks if the given shop has an active plan
            and return a boolean with the result
        """
        #Check if the payments are disabled, so in this case the plan is active (doesnt need one)
        if not ConfigService().is_active_billing():
            return True

        #Check if the user has a plan or if has an inactive plan
        current_plan = shop.current_plan()
        if not current_plan:
            return False

        return PlanService().is_active_plan(current_plan)

    def before_save(self, shop_model, request):
        """
            Override this method and add behaviour on before save shop
        """

        pass

    def before_install(self, request):
        """
            Override this method and add behaviour on before install
        """

        pass

    def post_install(self, request):
        """
            Override this method and add behaviour on post install
        """

        pass

    def get_upgrade_plan_url(self, shop):
        """
            Creates the shop plan and returns the confirmation url where
            the user should accept the billing.
        """

        if not ConfigService().is_active_billing():
            return False

        config = ConfigService().get_config()
        plan_config = config.plan_config
        response_data = PlanConfigService().confirm_data(shop, plan_config)

        return response_data["confirmation_url"]

    def find_app_charge(self, entity, shop_model, charge_id):

        return APIWrapper(shop_model, log=True).get(entity, charge_id)

    def activate_charge(self, entity, shop_model, charge_id):

        APIWrapper(shop_model, log=True).activate_charge(entity, charge_id)

    def upgrade_plan(self, shop_id, plan_config_id, charge_id):
        """
            Upgrades the plan for the current [shop_id] using the [plan_config_id].
            It also saved the [charge_id] on the plan model.
        """
        shop_model = self.get(id=shop_id)
        plan_config = PlanConfigService().get(id=plan_config_id)

        api_object = "application_charge" if plan_config.billing_type == "O" else "recurring_application_charge"
        charge = self.find_app_charge(api_object, shop_model, charge_id)

        if charge["status"] == "accepted":
            self.activate_charge(api_object, shop_model, charge["id"])
        elif charge["status"] == "declined":
            return False

        #Check if the charge is already activated
        charge = self.find_app_charge(api_object, shop_model, charge_id)

        if charge["status"] == "active":

            plan = PlanService().new(shop=shop_model)

            #copies all the attributes from the plan_config to the plan model
            for field in plan_config.update_fields():
                setattr(plan, field, getattr(plan_config, field, ""))

            plan.charge_id = charge_id
            plan.installed_at = datetime.utcnow()
            plan.save()
        else:
            return False

        return True



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

#from djangp_shopify import __version__
__version__ = "0.4.2"

__dir__ = os.path.dirname(os.path.abspath(__file__))
templates_dir = os.path.join(__dir__, "shopify_app", "management", "commands", "templates")
templates_files = [os.path.join(templates_dir, file) for file in os.listdir(templates_dir)]

setup(
    name='django-shopify',
    version=__version__,
    description='Django-Shopify generic app',
    author='Bootstrap Heroes Developers',
    author_email='jmg.utn@gmail.com',
    url='https://github.com/SocalProofit/django-shopify',
    classifiers=[
        "Framework :: Django",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Programming Language :: Python :: 2.7",
    ],
    packages=find_packages(exclude=["django_shopify"]),
    data_files=[
        (templates_dir, templates_files)
    ],
    include_package_data=True,
    install_requires=[
        "simplejson",
        "django_conventions",
        "requests",
    ],
    scripts=['bin/start_shopify_app'],
)



import csv


class Exporter(object):

    def csv(self, response, content):

        writer = csv.writer(response)
        for row in content:
            writer.writerow([value.encode("utf-8") for value in row])

        return response

    def export(self, format, response, content):

        return getattr(self, format)(response, content)


from django.template.loader import get_template
from django.template import Context, Template


def render(template, context):
    
    context = Context(context)
    return get_template(template).render(context)


def render_string(string, context):

    template = Template(string)
    context = Context(context)
    return template.render(context)